package jalview.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import jalview.analysis.*;


public class SeqCanvas extends JPanel
{
    ColourSchemeI globalColorScheme = null;
    Image             img;
    Graphics          gg;
    int               imgWidth;
    int               imgHeight;

    AlignViewport     av;
    int pady = 2;
    int oldstartx;
    int oldstarty;
    int oldendx;
    int oldendy;
    public boolean paintFlag = false;

    boolean showScores = false;
    boolean displaySearch = false;
    int [] searchResults = null;

    int chunkHeight;
    int chunkWidth;


    public SeqCanvas(AlignViewport av)
    {
	this.av         = av;
       setLayout(new BorderLayout());

    }

  public void drawScale(int startx, int endx,int charWidth, int charHeight,int ypos) {
      int scalestartx = startx - startx%10 + 10;

      gg.setColor(Color.black);

      for (int i=scalestartx;i < endx;i+= 10) {
	  String string = String.valueOf(i);
	  gg.drawString(string,(int)((i-startx-1)*charWidth),ypos+15 - charHeight*(2));
      }
  }


/**
 * Definitions of startx and endx (hopefully):
 * SMJS This is what I'm working towards!
 *   startx is the first residue (starting at 0) to display.
 *   endx   is the last residue to display (starting at 0).
 *   starty is the first sequence to display (starting at 0).
 *   endy   is the last sequence to display (starting at 0).
 * NOTE 1: The av limits are set in setFont in this class and
 * in the adjustment listener in SeqPanel when the scrollbars move.
 */

  public void paintComponent(Graphics g) {

    AlignmentI da = av.getAlignment();

    if (img == null ||
        imgWidth  !=  getWidth()  ||
        imgHeight !=  getHeight()
        || paintFlag)
    {

      imgWidth  = (getWidth() > 0 ? getWidth() : 1);
      imgHeight = (getHeight() > 0 ? getHeight() : 1);

      img = createImage(imgWidth,imgHeight);
      gg  = img.getGraphics();

      gg.setFont(av.getFont());

      oldstartx = -1;
      oldendx   = -1;
      oldstarty = -1;
      oldendy   = -1;
      paintFlag = false;
    }

    int startx = av.getStartRes();
    int starty = av.getStartSeq();

    int endx   = av.getEndRes();
    int endy   = av.getEndSeq();

    int charWidth  = av.getCharWidth();
    int charHeight = av.getCharHeight();

    chunkWidth  =   getWidth()/charWidth;
    chunkHeight =  (da.getHeight() + 2)*charHeight;

    av.setChunkHeight(chunkHeight);
    av.setChunkWidth(chunkWidth);

    int offy = av.getStartSeq();

    if (oldendx == -1) {
      fillBackground(gg,Color.WHITE,0,0,imgWidth,imgHeight);

     if (av.getWrapAlignment()) {
	  startx = (int)(offy/chunkWidth)*chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = offy%chunkHeight;
	  endy   = starty + da.getHeight();

	  int ypos     = 0;
	  int rowstart = starty;

	  if (starty == 0) {
	      ypos = 2*charHeight;
	  } else if (starty == 1) {
	      starty = 0;
	      ypos = charHeight;
	  }

	  if (endy > da.getHeight()) {
	      endy = da.getHeight();
	  }

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }

	  if (rowstart < 2) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	  }

	  drawPanel(gg,startx,endx,starty,endy,startx,starty,ypos);

	  if (rowstart == 0) {
	      ypos = ypos + chunkHeight;
	  } else if (rowstart == 1) {
	      ypos = ypos + chunkHeight;
	  } else {
	      ypos   = ypos + chunkHeight - rowstart*charHeight;
	  }

	  startx += chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = 0;

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }
	  // Draw the rest of the panels

	  while (ypos <= getHeight()) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	      drawPanel(gg,startx,endx,0,da.getHeight(),startx,starty,ypos);

	      ypos   += chunkHeight;
	      startx += chunkWidth;
	      endx   = startx + chunkWidth;

	      if (endy > da.getHeight()) {
		  endy = da.getHeight();
	      }

	      if (endx > da.getWidth()) {
		  endx = da.getWidth();
	      }

	  }
      }
      else
      {
	  drawPanel(gg,startx,endx,starty,endy,startx,starty,0);

	  oldstartx = startx;
	  oldendx   = endx;
	  oldstarty = starty;
	  oldendy   = endy;

      }
    }

    else if (oldstartx < startx)
    {
      // This is dragging horizontal scrollbar to the right

      int delx  = (startx - oldstartx)   * charWidth;
      int delx2 = (oldendx - startx + 1) * charWidth;

      gg.copyArea(delx,0,delx2,AlignmentUtil.getPixelHeight(starty,endy,charHeight),-delx,0);

      if (startx > oldendx)
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      else
	drawPanel(gg,oldendx+1,endx,starty,endy,startx,starty,0);


      oldstartx = startx;
      oldendx   = endx;

    } else if (oldstartx > startx)
    {
     // Horizontal scrollbar pulled to the left

      int delx  = (oldstartx - startx)    * charWidth;
      int delx2 = (endx - oldstartx +1)   * charWidth;

      gg.copyArea(0,0,delx2,AlignmentUtil.getPixelHeight(starty,endy,charHeight),delx,0);

      if (oldstartx > endx) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,oldstartx-1,starty,endy,startx,starty,0);
      }

      oldstartx = startx;
      oldendx   = endx;

    }  else if (oldstarty < starty) {
      // Vertical scrollbar down
      int dely  = AlignmentUtil.getPixelHeight(oldstarty,starty,charHeight);
      int dely2 = AlignmentUtil.getPixelHeight(starty,oldendy,charHeight);

      gg.copyArea(0,dely,(endx-startx+1)*charWidth,dely2,0,-dely);

      if (starty > oldendy) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,endx,oldendy,endy,startx,starty,0);
      }

      oldstarty = starty;
      oldendy   = endy;

    } else if (oldstarty > starty) {

      // Vertical scrollbar up
      int dely  = AlignmentUtil.getPixelHeight(endy,oldendy,charHeight);
      int dely2 = AlignmentUtil.getPixelHeight(oldstarty,endy,charHeight);

      gg.copyArea(0,0,(endx-startx+1)*charWidth,dely2,0,dely);

      if (oldstarty > endy) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,endx,starty,oldstarty,startx,starty,0);
      }

      oldstarty = starty;
      oldendy   = endy;
    }

    if ((oldendy -oldstarty) > (getWidth() / av.getCharWidth())) {
      System.out.println("LIMITS ERROR LIMITS ERROR");
      System.out.println("Corrds " + (oldendy-oldstarty) + " " + (getWidth()/av.getCharWidth()) + " " + getWidth() + " " + av.getCharWidth());
    }


  //  gg.setColor(Color.red);
  //  gg.drawRect( groupX, groupY, groupendX-groupX, groupendY-groupY );
   // gg.drawRect( groupX+1, groupY+1, groupendX-groupX-2, groupendY-groupY-2 );


    g.drawImage(img,0,0,this);

  }




  public void drawPanel(Graphics g,int x1,int x2, int y1, int y2,int startx, int starty,int offset) {

/*
    Startx and starty appear to be the same as x1, y1, unless you find different
      x1                                               x2
    y1**************************************************
      *                                                *
      *                                                *
      *                                                *
      *                                                *
      **************************************************y2

/*
    System.out.println("drawPanel called with g      = " + g);
    System.out.println("                      x1     = " + x1);
    System.out.println("                      x2     = " + x2);
    System.out.println("                      y1     = " + y1);
    System.out.println("                      y2     = " + y2);
    System.out.println("                      startx = " + startx);
    System.out.println("                      starty = " + starty);
*/

    g.setFont(av.getFont());
    int            charWidth  = av.getCharWidth();
    int            charHeight = av.getCharHeight();
    RendererI sr = av.getRenderer();


    /*Vector    pid    = av.getConsensus(false);
    Vector tmpseq = new Vector();
    for (int i = 0; i < av.getAlignment().getHeight(); i++)
	if (!av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
	    tmpseq.addElement(av.getAlignment().getSequenceAt(i));

    if (sr instanceof SequenceRenderer)
	pid    = AAFrequency.calculate(tmpseq,x1,x2);

    else if (sr instanceof GraphRenderer)
	pid = AAFrequency.calculatePID(av.getAlignment().getSequenceAt(0),
				       av.getAlignment().getSequences(),
				       av.getPIDWindow(),x1,x2);

*/

    if (y2 > starty && y1 < av.getEndSeq())
    {
       fillBackground(g,
                   Color.WHITE,
                   (x1-startx)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(starty,y1,av.getCharHeight()),
                   (x2-x1+1)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(y1,y2,av.getCharHeight()));
    }

    SequenceI nextSeq;
    SequenceGroup group=null;
    boolean newGroup=false;
    int sx, sy, ex, ey;
    int oldStartX=0,oldEndX=0,oldY=0;
    for (int i = y1 ; i < y2 ;i++)
    {
     nextSeq = av.getAlignment().getSequenceAt(i);
     if(group==null || group!=av.alignment.findGroup( nextSeq ))
        newGroup = true;
     else
        newGroup = false;

    group = av.alignment.findGroup( nextSeq );

    sr.drawSequence(g, nextSeq, group,x1,x2,
                 (x1 - startx) * charWidth,
                 offset + AlignmentUtil.getPixelHeight(starty, i, av.getCharHeight()),
                 charWidth,charHeight,null, i);


     if( group!=null )
     {

        g.setColor(Color.GREEN);

        sx = (group.getStartRes()-startx)*charWidth;
        sy = offset + AlignmentUtil.getPixelHeight(starty, i, av.getCharHeight());
        ex = (group.getEndRes()+1-startx)*charWidth;
        ey = offset + AlignmentUtil.getPixelHeight(starty, i+1, av.getCharHeight());


        if (newGroup)
        {
          g.drawLine(oldStartX, oldY, oldEndX, oldY); // Horizontal, bottom of old box
          g.drawLine(sx, sy, ex, sy); // Horizontal, top of new box
        }

        g.drawLine(sx, sy, sx, ey ); // vertical line, left hand side
        g.drawLine( ex,sy,ex,ey);    // vertical line, right hand side
        if (i == y2 - 1)// last line of alignment, seal the box
          g.drawLine(sx, ey, ex, ey); // Horizontal, bottom of old box

        oldStartX = sx;
        oldEndX = ex;
        oldY = ey;

     }

    }

    /// Highlight search Results once all sequences have been drawn
    if(displaySearch)
    {
      for(int r=0; r<searchResults.length; r+=3)
      {
        int searchSeq = searchResults[r];
        int searchStart = searchResults[r+1];
        int searchEnd = searchResults[r+2];

        if (searchSeq >= y1 && searchSeq <= y2)
        {
          SequenceRenderer ssr = (SequenceRenderer) sr;
          ssr.drawHighlightedText(g, av.getAlignment().getSequenceAt(searchSeq),
                                  searchStart,
                                  searchEnd,
                                  (searchStart - startx) * charWidth,
                                  offset +
                                  AlignmentUtil.getPixelHeight(starty, searchSeq,
              charHeight),
                                  charWidth,
                                  charHeight);
        }
      }
    }




    /*bg.drawImage(image,0,0,null);
  int size=width*height;
  int pixel, b, x,y;
  if(value!=0 )
  {
    if(painted!=null)
    {
     for(int i=0; i<size; i++)
     {
      if(painted[i] == value)
      {
         x = (i%width);
         y = (i-x)/width;
         pixel = buf.getRGB(x,y);
         b = pixel & 0x000000ff;
         pixel = (105<<24) | (0<<16)+32 | (0<<8)+32 | b;
         buf.setRGB(x,y,pixel);
     }
     }

    }
    else
    {
     for(int i=0; i<size; i++)
     {
      if(paintedShort[i] == value)
      {
       x = (i%width);
       y = (i-x)/width;
       pixel = buf.getRGB(x,y);
       b = pixel & 0x000000ff;
       pixel = (105<<24) | (0<<16) | (0<<8) | b;
       buf.setRGB(x,y,pixel);
     }
     }
    }

   }
   oldvalue = value;
   return (Image)buf;
*/

  }

 // public int groupX, groupY, groupendX, groupendY;

  public void fillBackground(Graphics g,Color c, int x1,int y1,int width,int height) {
    g.setColor(c);
    g.fillRect(x1,y1,width,height);
  }

  public int getChunkWidth() {
    return chunkWidth;
  }

  public void highlightSearchResults(int [] results)
  {
    // results are in the order sequence, startRes, endRes
    if(results==null)
      displaySearch = false;
    else
      displaySearch = true;

    searchResults = results;

    paintFlag = true;
    repaint();
  }


}
